#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_path_1 = __importDefault(require("node:path"));
const node_util_1 = require("node:util");
const typescript_1 = __importDefault(require("typescript"));
const help_1 = require("./help");
const config_1 = require("./util/config");
const path_1 = require("./util/path");
const reporters_1 = __importDefault(require("./reporters"));
const _1 = require(".");
const { values: { help, cwd: cwdArg, config: configFilePath = 'knip.json', include = [], exclude = [], dev: isDev = false, 'no-progress': noProgress = false, reporter = 'symbols', jsdoc = [], 'max-issues': maxIssues = '0', }, } = (0, node_util_1.parseArgs)({
    options: {
        help: { type: 'boolean' },
        config: { type: 'string', short: 'c' },
        cwd: { type: 'string' },
        include: { type: 'string', multiple: true },
        exclude: { type: 'string', multiple: true },
        dev: { type: 'boolean' },
        'max-issues': { type: 'string' },
        'no-progress': { type: 'boolean' },
        reporter: { type: 'string' },
        jsdoc: { type: 'string', multiple: true },
    },
});
if (help) {
    (0, help_1.printHelp)();
    process.exit(0);
}
const cwd = process.cwd();
const workingDir = cwdArg ? node_path_1.default.resolve(cwdArg) : cwd;
const isShowProgress = noProgress === false ? process.stdout.isTTY && typeof process.stdout.cursorTo === 'function' : !noProgress;
const printReport = reporter in reporters_1.default ? reporters_1.default[reporter] : require(node_path_1.default.join(workingDir, reporter));
const main = async () => {
    const localConfigurationPath = await (0, path_1.findFile)(workingDir, configFilePath);
    const manifestPath = await (0, path_1.findFile)(workingDir, 'package.json');
    if (!manifestPath || !localConfigurationPath) {
        const location = workingDir === cwd ? 'current directory' : `${node_path_1.default.relative(cwd, workingDir)} or up.`;
        console.error(`Unable to find ${configFilePath} (or package.json#knip) in ${location}\n`);
        (0, help_1.printHelp)();
        process.exit(1);
    }
    const localConfiguration = require(localConfigurationPath);
    const manifest = require(manifestPath);
    const resolvedConfig = (0, config_1.resolveConfig)(manifest.knip ?? localConfiguration, cwdArg);
    if (!resolvedConfig) {
        (0, help_1.printHelp)();
        process.exit(1);
    }
    const report = (0, config_1.resolveIncludedIssueGroups)(include, exclude, resolvedConfig);
    const tsConfigFile = await (0, path_1.findFile)(workingDir, 'tsconfig.json');
    if (!tsConfigFile) {
        const location = workingDir === cwd ? 'current directory' : `${node_path_1.default.relative(cwd, workingDir)} or up.`;
        console.error(`Unable to find ${tsConfigFile} or package.json in ${location}\n`);
        (0, help_1.printHelp)();
        process.exit(1);
    }
    const tsConfig = typescript_1.default.readConfigFile(tsConfigFile, typescript_1.default.sys.readFile);
    const tsConfigPaths = tsConfig.config.compilerOptions?.paths
        ? Object.keys(tsConfig.config.compilerOptions.paths).map(p => p.replace(/\*/g, '**'))
        : [];
    if (tsConfig.error) {
        console.error(`An error occured when reading ${node_path_1.default.relative(cwd, tsConfigFile)}.\n`);
        (0, help_1.printHelp)();
        process.exit(1);
    }
    const config = {
        workingDir,
        report,
        dependencies: Object.keys(manifest.dependencies ?? {}),
        devDependencies: Object.keys(manifest.devDependencies ?? {}),
        isDev: resolvedConfig.dev ?? isDev,
        tsConfigPaths,
        isShowProgress,
        jsDocOptions: {
            isReadPublicTag: jsdoc.includes('public'),
        },
        ...resolvedConfig,
    };
    const { issues, counters } = await (0, _1.run)(config);
    printReport({ issues, workingDir, config });
    const reportGroup = report.files ? 'files' : Object.keys(report).find(key => report[key]);
    const counterGroup = reportGroup === 'unlisted' ? 'unresolved' : reportGroup;
    if (counterGroup) {
        const count = counters[counterGroup];
        if (count > Number(maxIssues))
            process.exit(count);
    }
};
main();
